//go:generate protoc --go_out=plugins=grpc:. test.proto

package main

import(
	"context"
	"log"
	"math/rand"
	"net"
	"os"

	"google.golang.org/grpc"
)

const(
	NR_ITERS = 64 * 1024
)

func main() {
	if len(os.Args) != 2 {
		use()
	}

	if os.Args[1] == "server" {
		server()
	} else if os.Args[1] == "client" {
		client()
	} else {
		use()
	}
}

type ArithServiceImpl struct {
	UnimplementedArithServiceServer
}

func (self *ArithServiceImpl) Sum(ctx context.Context, req *SumReq) (*SumResp, error) {
	sum := int64(0)
	for _, x := range req.Xs {
		sum += x
	}

	resp := SumResp{Sum: sum}
	return &resp, nil
}

func server() {
	addr := "127.0.0.1:9091"
	l, err := net.Listen("tcp", addr)
	if err != nil {
		log.Fatalf("net.Listen(%s) failed: %s\n", addr, err)
	}

	server := grpc.NewServer()
	RegisterArithServiceServer(server, &ArithServiceImpl{})

	err = server.Serve(l)
	if err != nil {
		log.Fatalf("grpc.Serve() failed: %s\n", err)
	}
}

func makeSumRequest() *SumReq {
	nr := rand.Intn(2048)
	xs := make([]int64, nr)
	for i := 0; i < nr; i++ {
		xs[i] = int64(i*100);
	}
	return &SumReq{Xs: xs}
}

func client() {
	addr := "127.0.0.1:9091"
	conn, err := grpc.Dial(addr, grpc.WithInsecure(), grpc.WithBlock())
	if err != nil {
		log.Fatalf("grpc.Dial(%s) failed: %s\n", addr, err)
	}

	testClient := NewArithServiceClient(conn)
	ctx := context.Background()

	for i := int64(1); i <= NR_ITERS; i++ {
		_, err := testClient.Sum(ctx, makeSumRequest())
		if err != nil {
			log.Fatalf("failed to send a request: %s\n", err)
		}
	}
}

func use() {
	log.Fatalf("use: test server|client\n")
}
